/*
Tecnicas Heuristicas para resolucao do Problema do Caixeiro Viajante
Autor: Marcone Jamilson Freitas Souza
Local: Departamento de Computacao - Universidade Federal de Ouro Preto
Homepage: www.decom.ufop.br/prof/marcone
Data: 06-04-2005
*/

//---------------------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <limits.h>
#include <time.h>
#include "Util.h"
#include "Arquivos.h"
#include "Estruturas.h"
#include "Menus.h"
#include "Construcao.h"
#include "Listas.h"
#include "Descida.h"
#include "SA.h"
#include "BT.h"
#include "VNS.h"
#include "GRASP.h"
#include "AG.h"

//---------------------------------------------------------------------------


int main(int argc, char* argv[])
{
  int n,                    // numero de cidades
      x_dep, y_dep,         // localizacao do deposito (x,y)
      cap,                  // capacidade dos veiculos
      *s,                   // vetor solucao corrente
      *demanda;             // vetor de demanda de cada cidade
  float **d,                // matriz de distancias entre as cidades
        fo;                 // funcao objetivo corrente
  clock_t inicio_CPU,       // clock no inicio da aplicacao do metodo
        fim_CPU;            // clock no final da aplicacao do metodo



  obter_parametros_prob("c50info.txt", &n, &cap, &x_dep, &y_dep);
  n = n + 1;
  s = cria_vetor(n); // vetor solucao
  demanda = cria_vetor(n); // vetor de demanda de cada cidade
  d = cria_matriz_float(n, n); // matriz de distancias

  le_arq_matriz("c50.txt", n, x_dep, y_dep, d, demanda);

  int escolha = 0;
  do {
    escolha = menu_principal();
    switch (escolha) {
    case 1: /* Geracao de uma solucao inicial */
           switch(menu_solucao_inicial()) {
           case 1: /* Geracao gulosa de uma solucao inicial via metodo do vizinho mais proximo */
                 constroi_solucao_gulosa_vizinho_mais_proximo(n,s,d);
                 fo = calcula_fo(n, s, d);
                 printf("\nSolucao construida de forma gulosa:\n");
                 imprime_rota(s, n);
                 printf("Funcao objetivo = %f\n",fo);
	         break;
           case 2: /* Geracao parcialmente gulosa de uma solucao inicial via metodo do vizinho mais proximo */
                 constroi_solucao_parcialmente_gulosa_vizinho_mais_proximo(n,s,d,0.05);
                 fo = calcula_fo(n, s, d);
                 printf("\nSolucao construida de forma parcialmente gulosa pelo Metodo do Vizinho Mais Proximo :\n");
                 imprime_rota(s, n);
                 printf("Funcao objetivo = %f\n",fo);
	         break;
           case 3: /* Geracao gulosa de uma solucao inicial via metodo da insercao mais barata */
                 //constroi_solucao_insercao_mais_barata(n,s,d);
                 constroi_solucao_gulosa_insercao_mais_barata(n, s, d);
                 fo = calcula_fo(n, s, d);
                 printf("\nSolucao construida de forma gulosa pelo Metodo da Insercao Mais Barata:\n");
                 imprime_rota(s, n);
                 printf("Funcao objetivo = %f\n",fo);
	         break;
           case 4: /* Geracao parcialmente gulosa de uma solucao inicial via insercao mais barata */
                 printf("\n Ainda nao implementado.\n");
	         break;
           case 5: /* Geracao aleatria de uma solucao inicial */
                 srand(1000); // pega o numero 1000 como semente de numeros aleatorios
                 //srand((unsigned) time(NULL)); // pega a hora do relogio como semente
                 constroi_solucao_aleatoria(n,s,d);
                 fo = calcula_fo(n, s, d);
                 printf("\nSolucao construida de forma aleatoria:\n");
                 imprime_rota(s, n);
                 printf("Funcao objetivo = %f\n",fo);
	         break;
           }
           break;
    case 2: /* Descida */
           inicio_CPU = clock();
           fo = descida(n, s, d);
           fim_CPU = clock();
           printf("\nSolucao construida usando o Metodo da Descida:\n");
           imprime_rota(s, n);
           printf("\nMelhor fo encontrada = %f \n",fo);
           printf("Tempo de CPU = %f segundos\n",(fim_CPU - inicio_CPU)/CLK_TCK);
           break;
    case 3: /* Descida Randmica */
           fo = descida_randomica(n, s, d, 2500);
           printf("\nSolucao construida usando o Metodo da Descida Randomica:\n");
           imprime_rota(s, n);
           printf("\nMelhor fo encontrada = %f \n",fo);
           printf("Tempo de CPU = %f segundos\n",(fim_CPU - inicio_CPU)/CLK_TCK);
           break;
    case 4: /* Descida Parcial */
           inicio_CPU = clock();
           fo = descida_parcial(n, s, d, 50);
           fim_CPU = clock();
           printf("\nSolucao construida usando o Metodo da Descida:\n");
           imprime_rota(s, n);
           printf("\nMelhor fo encontrada = %f \n",fo);
           printf("Tempo de CPU = %f segundos\n",(fim_CPU - inicio_CPU)/CLK_TCK);
           break;
    case 5: /* Simulated Annealing */
           float temperatura_inicial;
           int k;
           k = 10;
           temperatura_inicial = calcula_temperatura_inicial(n,s,d,1.1,0.95,k*n);
           inicio_CPU = clock();
           fo = SA(n,s,d,0.995,temperatura_inicial,0.01,k*n);
           fim_CPU = clock();
           printf("\nSolucao construida usando o Metodo Simulated Annealing:\n");
           imprime_rota(s, n);
           printf("\nMelhor fo encontrada = %f \n",fo);
           printf("Tempo de CPU = %f segundos\n",(fim_CPU - inicio_CPU)/CLK_TCK);
           break;
    case 6: /* Busca Tabu */
           inicio_CPU = clock();
           fo = BT(n,s,d,10,300);
           fim_CPU = clock();
           printf("\nSolucao construida usando o Metodo Busca Tabu:\n");
           imprime_rota(s, n);
           printf("\nMelhor fo encontrada = %f \n",fo);
           printf("Tempo de CPU = %f segundos\n",(fim_CPU - inicio_CPU)/CLK_TCK);
           break;
    case 7: /* GRASP */
           switch(menu_GRASP()) {
           case 1: /* Geracao Parcialmente gulosa de uma solucao inicial via metodo do vizinho mais proximo */
                 srand(1000); // pega o numero 1000 como semente de numeros aleatorios
                 //srand((unsigned) time(NULL)); // pega a hora do relogio como semente
                 inicio_CPU = clock();
                 // GRASP(int n, int *s, float **d, float alpha, int GRASP_max, int tipo_construcao)
                 fo = GRASP(n,s,d,0.10,100,1);
                 fim_CPU = clock();
                 printf("\nSolucao construida usando o Metodo GRASP baseado no Vizinho mais proximo:\n");
                 imprime_rota(s, n);
                 printf("\nMelhor fo encontrada = %f \n",fo);
                 printf("Tempo de CPU = %f segundos\n",(fim_CPU - inicio_CPU)/CLK_TCK);
                 break;
           case 2: /* Geracao parcialmente gulosa de uma solucao inicial via metodo da insercao mais barata */
                 srand(1000); // pega o numero 1000 como semente de numeros aleatorios
                 //srand((unsigned) time(NULL)); // pega a hora do relogio como semente
                 inicio_CPU = clock();
                 // GRASP(int n, int *s, float **d, float alpha, int GRASP_max, int tipo_construcao)
                 fo = GRASP(n,s,d,0.10,100,2);
                 fim_CPU = clock();
                 printf("\nSolucao construida usando o Metodo GRASP baseado em Insercao mais barata:\n");
                 imprime_rota(s, n);
                 printf("\nMelhor fo encontrada = %f \n",fo);
                 printf("Tempo de CPU = %f segundos\n",(fim_CPU - inicio_CPU)/CLK_TCK);
	         break;
           }
           break;
    case 8: /* VND */
           printf("\n VND ainda nao implementado ... \n");
           break;
    case 9: /* VNS */
           inicio_CPU = clock();
           fo = VNS(n,s,d,3,8);
           fim_CPU = clock();
           printf("\nSolucao construida usando o Metodo VNS:\n");
           imprime_rota(s, n);
           printf("\nMelhor fo encontrada = %f \n",fo);
           printf("Tempo de CPU = %f segundos\n",(fim_CPU - inicio_CPU)/CLK_TCK);
           break;
    case 10: /* Algoritmos Geneticos */
           switch(menu_AG()) {
           case 1: /* Algoritmos Geneticos usando operador OX */
                 srand(1500);
                 inicio_CPU = clock();
                 //AG(n,s,d,nind,max_desvio, prob_crossover,prob_mutacao,tipo_operador);
                 fo = AG(n,s,d,200,0.01,0.80,0.01,1);
                 fim_CPU = clock();
                 printf("\nSolucao construida usando Algoritmos Geneticos:\n");
                 imprime_rota(s, n);
                 printf("\nMelhor fo encontrada = %f \n",fo);
                 printf("Tempo de CPU = %f segundos\n",(fim_CPU - inicio_CPU)/CLK_TCK);
                 break;
           case 2: /* Algoritmos Geneticos usando operador ERX */
                 srand(1500);
                 inicio_CPU = clock();
                 //AG(n,s,d,nind,max_desvio, prob_crossover,prob_mutacao,tipo_operador);
                 fo = AG(n,s,d,200,0.01,0.80,0.01,2);
                 fim_CPU = clock();
                 printf("\nSolucao construida usando Algoritmos Geneticos:\n");
                 imprime_rota(s, n);
                 printf("\nMelhor fo encontrada = %f \n",fo);
                 printf("Tempo de CPU = %f segundos\n",(fim_CPU - inicio_CPU)/CLK_TCK);
                 break;
           }
           break;
    default:
          break;
    }
  } while (escolha != 0);


  libera_vetor(s);
  libera_vetor(demanda);
  libera_matriz_float(d, n);
  return 0;
}
//---------------------------------------------------------------------------
